require 'net/http'
require 'uri'

class InstantPaymentNotificationController < OrderVerificationController

    # This will map the incoming IPN transaction types to methods, allowing
    # for single methods that handle multiple transaction types. In addition,
    # this allows us to disregard the majority of transaction types that we
    # do not care about.
    TRANSACTION_METHODS = Hash.new
    TRANSACTION_METHODS["chargeback"] = "notify_admin" # Chargeback
    TRANSACTION_METHODS["adjustment"] = "notify_admin"
    TRANSACTION_METHODS["merch_pmt"] = "notify_admin"
    TRANSACTION_METHODS["new_case"] = "notify_admin"
    TRANSACTION_METHODS["recurring_payment"] = "unexpected_payment_source"
    TRANSACTION_METHODS["send_money"] = "unexpected_payment_source"
    TRANSACTION_METHODS["subscr_payment"] = "unexpected_payment_source"
    TRANSACTION_METHODS["virtual_terminal"] = "unexpected_payment_source"
    TRANSACTION_METHODS["web_accept"] = "unexpected_payment_source"
    TRANSACTION_METHODS["cart"] = "verify_order"
    TRANSACTION_METHODS["express_checkout"] = "verify_order"
            
    def notify

        # We'll need to send this back unaltered in order to
        # acknowledge the receipt of the message
        msg = request.query_string

        logger.info "Query String: " + msg

        # Make sure this is a valid message by sending it back
        # to paypal
        url = URI.parse("http://" + Util.paypal_root + "/cgi-bin/webscr?cmd=_notify-validate&" + msg)
        res = Net::HTTP.get_response(url)
        res_val = CGI.unescape(res.body)

        logger.info "Response:" + res_val

        if res_val == "INVALID"
            return
        end

        # We'll use the hash-ified version of the query string
        # for all of our processing
        params = request.parameters

        dispatch(params)
    end

   
    private

    def dispatch(params)

        logger.info "IPN transaction type: " + params["txn_type"]

        if params["txn_type"] == ""
            params["txn_type"] = "chargeback"
        end

        # If there is no method defined for a transaction type, note it.
        # Otherwise, call the associated method.
        if TRANSACTION_METHODS[params["txn_type"]].nil?
            ApplicationMailer.deliver_parameter_email "IPN - MISSING HANDLER", params
            logger.error "No callable action found for IPN transaction type of " + params["txn_type"]
        else
            # Make sure that
            send(TRANSACTION_METHODS[params["txn_type"]], params)
        end
    end
end
